package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.util.common.UUIDUtil;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.bo.EwbAnalysisSettingBo;
import com.nationrel.weibull.domain.bo.EwbAnalysisSettingSaveBo;
import com.nationrel.weibull.domain.entity.EwbAnalysisDisWizard;
import com.nationrel.weibull.domain.vo.*;
import com.nationrel.weibull.service.EwbAnalysisDisWizardService;
import com.nationrel.weibull.service.EwbAnalysisService;
import com.nationrel.weibull.service.EwbAnalysisSettingService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * @author Aimee on 2020/9/14.
 */
@Api(tags = "寿命数据分析-分析部分")
@Slf4j
@Validated
@RestController
@RequestMapping("/analysis")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisController {

    @Value("${redisson.wait.time}")
    private int redissonWaitTime;
    @Value("${redisson.last.time}")
    private int redissonLastTime;
    //    private final RedissonClient redissonClient;
    private final EwbAnalysisService ewbAnalysisService;
    private final EwbAnalysisSettingService ewbAnalysisSettingService;
    private final EwbAnalysisDisWizardService ewbAnalysisDisWizardService;

    @ApiOperation(value = "分析列表", notes = "寿命数据分析")
    @GetMapping("/list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisName", value = "分析名称", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse<List<EwbAnalysisListVo>> analysisList(HttpServletRequest request,
                                                                @RequestParam(required = false) String analysisName,
                                                                @RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        List<EwbAnalysisListVo> result = ewbAnalysisService.listByPage(wbId, analysisName, userInfo);
        return ServerResponse.createBySuccess(result);
    }

    @ApiOperation(value = "获取timeForRank", notes = "获取timeForRank")
    @GetMapping("/timeForRank")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisName", value = "分析名称", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse<String> timeForRank(@RequestParam@NotBlank(message = "DataIdNotEmpty") String dataId) {
        return ewbAnalysisService.timeForRank(dataId);
    }

    @ApiOperation(value = "分析创建", notes = "寿命数据分析")
    @PostMapping("/create")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse analysisCreate(HttpServletRequest request, @RequestBody @Valid EwbAnlaysisCreateVo param) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        String analysisId = UUIDUtil.getUuid();
        EwbAnalysisReqVo analysis = ewbAnalysisService.createAnalysis(userInfo, param.getWbId(), analysisId, param.getAnalysisName(), param.getObjectCode(), param.getObjectName(), param.getDescription());
        return ServerResponse.createBySuccess(analysis);
    }

    @ApiOperation(value = "分析编辑保存", notes = "寿命数据分析")
    @PostMapping("/save")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse analysisSave(HttpServletRequest request, @RequestBody @Valid EwbAnalysisSaveVo param) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return ewbAnalysisService.editProject(userInfo, param);
    }

    @ApiOperation(value = "无数据时直接画图", notes = "寿命数据分析")
    @PostMapping("/calcNoData")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse calcNoData(HttpServletRequest request, @RequestBody @Valid EwbCalcResultVo param) {
        return ewbAnalysisService.calcNoData(param);
    }

    @ApiOperation(value = "分析删除", notes = "寿命数据分析")
    @GetMapping("/del")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisId", value = "分析ID", dataType = "String", paramType = "query")
    })
    public ServerResponse analysisCreate(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String analysisId) {
        return ewbAnalysisService.delAnalysis(analysisId);
    }

    @ApiOperation(value = "获取参数估计", notes = "参数估计")
    @PostMapping("/getAnalysis-setting")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getAnalysisSetting(@RequestBody EwbAnalysisSettingBo param) {
        EwbAnalysisSettingVo setting = ewbAnalysisSettingService.getSetting(param);
        return ServerResponse.createBySuccess(setting);
    }

    @ApiOperation(value = "保存分析设置", notes = "分析设置")
    @PostMapping("/analysis-setting-save")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse saveAnalysisSetting(HttpServletRequest request, @RequestBody EwbAnalysisSettingSaveBo param) {
        HeaderUserInfo user = UserInfoUtil.parseHeader(request);
        ewbAnalysisSettingService.saveSetting(user, param);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "分析复制", notes = "寿命数据分析")
    @GetMapping("/copy")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisId", value = "分析ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "dataId", value = "数据ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "settingId", value = "设置ID", dataType = "String", paramType = "query")
    })
    public ServerResponse analysisCopy(@NotBlank(message = "AnalysisIdNotEmpty") String analysisId,
                                       @NotBlank(message = "DataIdNotEmpty") String dataId,
                                       @NotBlank(message = "SetIdNotEmpty") String settingId) {
        ewbAnalysisService.analysisCopy(analysisId, dataId, settingId);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "获取分布向导列表", notes = "寿命数据分析")
    @GetMapping("/distribution-wizard")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisId", value = "分析ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "dataId", value = "数据ID", dataType = "String", paramType = "query")
    })
    public ServerResponse distributionWizard(@RequestParam(required = false) String dataId) {
        List<EwbAnalysisDisWizard> result = ewbAnalysisDisWizardService.getDistributionWizard(dataId);
        return ServerResponse.createBySuccess(result);
    }

    @ApiOperation(value = "计算分布向导", notes = "寿命数据分析")
    @PostMapping("/distribution-wizard-calc")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse distributionWizardCalc(HttpServletRequest request, @RequestBody EwbDistributionVo param) {
        HeaderUserInfo user = UserInfoUtil.parseHeader(request);
        ewbAnalysisDisWizardService.getDistributionWizardCalc(param, user);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "计算数据", notes = "计算数据")
    @PostMapping("/calcResult")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse calcResult(HttpServletRequest request, @RequestBody EwbCalcResultVo param) throws Exception {
        return ewbAnalysisDisWizardService.calcResult(param, UserInfoUtil.parseHeader(request));
    }

    @ApiOperation(value = "选择分布向导", notes = "寿命数据分析")
    @PostMapping("/distribution-wizard-select")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse distributionWizardSelect(HttpServletRequest request, @RequestBody EwbDistributionSelectVo param) {
        HeaderUserInfo user = UserInfoUtil.parseHeader(request);
        ewbAnalysisDisWizardService.distributionWizardSelect(param, user);
        return ServerResponse.createBySuccess();
    }

//    @ApiOperation(value = "分析编辑保存", notes = "寿命数据分析")
//    @PostMapping("/save")
//    @ApiImplicitParams(value = {
//            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
//    })
//    public ServerResponse analysisSave(HttpServletRequest request, @RequestBody @Valid EwbAnalysisSaveVo param) {
//        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
//        RLock rLock = null;
//        boolean isLock = false;
//        try {
//            rLock = redissonClient.getLock(param.getAnalysisId());
//            isLock = rLock.tryLock(redissonWaitTime, redissonLastTime, TimeUnit.MILLISECONDS);
//            if (isLock) {
//                return ewbAnalysisService.editProject(userInfo, param);
//            } else {
//                //其他人正在操作，请稍后处理!
//                return ServerResponse.createByErrorMessage(EwbParamterCheckEnum.getLanguageMessageByKey(EwbParamterCheckEnum.其他人正在操作数据, UserInfoUtil.parseLK(request));
//            }
//        } catch (Exception e) {
//            log.error("节点删除 异常：%s", e);
//            return ServerResponse.createByErrorMessage(EwbParamterCheckEnum.getLanguageMessageByKey(EwbParamterCheckEnum.其他人正在操作数据, UserInfoUtil.parseLK(request));
//        } finally {
//            if (isLock) {
//                rLock.unlock();//释放Redisson锁
//            }
//        }
//    }
}
