package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.entity.EwbAnalysisSolveDataCreateVo;
import com.nationrel.weibull.domain.entity.EwbAnalysisSolveDataUpdateVo;
import com.nationrel.weibull.service.EwbAnalysisSolveDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

@Api(tags = "可靠性测试设计 历史数据")
@RestController
@RequestMapping("/solveData")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisSolveDataController {

    private final EwbAnalysisSolveDataService solveDataService;

    @ApiOperation(value = "列表")
    @GetMapping
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "solveName", value = "计算名称"),
            @ApiImplicitParam(name = "pageIndex", value = "当前页码"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数")
    })
    public ServerResponse list(HttpServletRequest request,
                               @RequestParam(required = false) String solveName,
                               @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                               @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return solveDataService.listByPage(userInfo, solveName, pageIndex, pageSize);
    }

    @ApiOperation(value = "根据id查询")
    @GetMapping("/{id}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "solveId", value = "计算ID")
    })
    public ServerResponse selectOne(@PathVariable(value = "id") String solveId) {
        return solveDataService.selectOneBySolveId(solveId);
    }

    @ApiOperation(value = "创建单个")
    @PostMapping
    public ServerResponse create(HttpServletRequest request, @RequestBody @Valid EwbAnalysisSolveDataCreateVo createVo) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return solveDataService.create(userInfo, createVo);
    }

    @ApiOperation(value = "根据id更新")
    @PutMapping
    public ServerResponse update(HttpServletRequest request, @RequestBody @Valid EwbAnalysisSolveDataUpdateVo updateVo) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return solveDataService.update(userInfo, updateVo);
    }

    @ApiOperation(value = "根据id删除")
    @DeleteMapping("/{id}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "solveId", value = "计算ID")
    })
    public ServerResponse delete(@PathVariable(value = "id") String solveId) {
        return solveDataService.deleteBySolveId(solveId);
    }
}
