package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.entity.EwbInspectInterval;
import com.nationrel.weibull.domain.entity.EwbReplaceInterval;
import com.nationrel.weibull.domain.vo.EwbCostOptimizeCreateVo;
import com.nationrel.weibull.service.EwbCostOptimizeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

@Api(tags = "成本优化分析")
@Validated
@RestController
@RequestMapping("/cost")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbCostOptimizeController {

    private final EwbCostOptimizeService optimizeService;

    @ApiOperation(value = "更换成本创建", notes = "成本优化分析")
    @PostMapping("/replace")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse createReplace(HttpServletRequest request, @RequestBody @Valid EwbCostOptimizeCreateVo param) {
        return ServerResponse.createBySuccess(optimizeService.createReplace(param, UserInfoUtil.parseHeader(request)));
    }

    @ApiOperation(value = "更换成本创建", notes = "成本优化分析")
    @GetMapping("/replace")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse replaceList(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return optimizeService.selectReplace(wbId);
    }

    @ApiOperation(value = "获取单个更换成本", notes = "成本优化分析")
    @GetMapping("/replace/{replaceIntervalId}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getReplace(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String replaceIntervalId) {
        return optimizeService.getReplace(replaceIntervalId);
    }

    @ApiOperation(value = "删除单个更换成本", notes = "成本优化分析")
    @DeleteMapping("/replace/{replaceIntervalId}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse deleteReplace(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String replaceIntervalId) {
        return optimizeService.deleteReplace(replaceIntervalId);
    }

    @ApiOperation(value = "更换成本名称修改", notes = "成本优化分析")
    @PutMapping("/replace")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse deleteReplace(@RequestBody @Valid EwbReplaceInterval replaceInterval) {
        return optimizeService.editReplace(replaceInterval);
    }

    @ApiOperation(value = "检测成本创建", notes = "成本优化分析")
    @PostMapping("/inspect")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse createInspect(HttpServletRequest request, @RequestBody @Valid EwbCostOptimizeCreateVo param) {
        return ServerResponse.createBySuccess(optimizeService.createInspect(param, UserInfoUtil.parseHeader(request)));
    }

    @ApiOperation(value = "检测成本列表", notes = "成本优化分析")
    @GetMapping("/inspect")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse inspectList(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return optimizeService.selectInspect(wbId);
    }

    @ApiOperation(value = "获取单个更换成本", notes = "成本优化分析")
    @GetMapping("/inspect/{inspectIntervalId}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getInspect(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String inspectIntervalId) {
        return optimizeService.getInspect(inspectIntervalId);
    }

    @ApiOperation(value = "删除单个更换成本", notes = "成本优化分析")
    @DeleteMapping("/inspect/{inspectIntervalId}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse deleteInspect(@PathVariable @NotBlank(message = "AnalysisIdNotEmpty") String inspectIntervalId) {
        return optimizeService.deleteInspect(inspectIntervalId);
    }

    @ApiOperation(value = "更换成本名称修改", notes = "成本优化分析")
    @PutMapping("/inspect")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse editInspect(@RequestBody @Valid EwbInspectInterval inspectInterval) {
        return optimizeService.editInspect(inspectInterval);
    }

}
