package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.util.common.UUIDUtil;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.general.domain.enums.common.EwbConstants;
import com.nationrel.weibull.domain.vo.*;
import com.nationrel.weibull.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.io.IOException;

/**
 * 退化分析
 */
@Api(tags = "退化分析")
@Validated
@RestController
@RequestMapping("/degradation")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbDegradationController {

    private final EwbDaInfoService ewbDaInfoService;
    private final EwbDaDataDetailService ewbDaDataDetailService;
    private final EwbDaSettingService ewbDaSettingService;
    private final EwbAnalysisService ewbAnalysisService;
    private final EwbAnalysisSettingService ewbAnalysisSettingService;
    private final EwbAnalysisDataDetailService ewbAnalysisDataDetailService;

    @ApiOperation(value = "退化创建", notes = "退化分析")
    @PostMapping("/create")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse create(HttpServletRequest request, @RequestBody @Valid EwbDegradationCreateVo param) {
        String daId = ewbDaInfoService.create(param.getName(), param.getWbId(), UserInfoUtil.parseHeader(request));
        return ServerResponse.createBySuccess(daId);
    }

    @ApiOperation(value = "退化列表", notes = "退化分析")
    @GetMapping("/list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse list(HttpServletRequest request, @NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return ServerResponse.createBySuccess(ewbDaInfoService.list(UserInfoUtil.parseHeader(request), wbId));
    }

    @ApiOperation(value = "退化名称修改-删除(修改名称传退化名称，删除时del_flag=1)", notes = "退化分析")
    @PostMapping("/edit")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse edit(HttpServletRequest request, @RequestBody @Valid EwbDaInfoSaveVo param) {
        ewbDaInfoService.edit(UserInfoUtil.parseHeader(request), param);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "数据列表", notes = "退化分析")
    @GetMapping("/data-list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    public ServerResponse dataList(
            @RequestParam String daId,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "100000000") int pageSize) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.dataList(daId, pageIndex, pageSize));
    }

    @ApiOperation(value = "获取分析结果列表", notes = "分析数据")
    @GetMapping("/getResultList")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getResultList(@NotBlank(message = "") String daId) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getResultList(daId));
    }

    @ApiOperation(value = "数据保存", notes = "退化分析")
    @PostMapping("/data-save")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse dataList(HttpServletRequest request, @RequestBody @Valid EwbDaDataDetailSaveVo param) {
        ewbDaDataDetailService.dataSave(UserInfoUtil.parseHeader(request), param);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "获取设置", notes = "退化分析")
    @GetMapping("/get-setting")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    public ServerResponse getSetting(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaSettingService.getSetting(daId));
    }

    @ApiOperation(value = "保存设置", notes = "退化分析")
    @PostMapping("/save-setting")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    public ServerResponse saveSetting(HttpServletRequest request, @RequestBody EwbDaSettingVo param) {
        ewbDaSettingService.update(UserInfoUtil.parseHeader(request).getUserId(), param);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "分析", notes = "退化分析")
    @GetMapping("/analysis")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    public ServerResponse analysis(HttpServletRequest request, @NotBlank(message = "DataIdNotEmpty") String daId, Double xmin, Double xmax, Double ymin, Double ymax) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.analysis(UserInfoUtil.parseHeader(request).getUserId(), daId, xmin, xmax, ymin, ymax, UserInfoUtil.parseHeader(request).getLanguageKey()));
    }

    @ApiOperation(value = "导入", notes = "退化分析")
    @PostMapping("/importData")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse importData(HttpServletRequest request, @RequestParam("file") MultipartFile file, @RequestParam("daId") String daId) throws Exception {
        ewbDaDataDetailService.importData(UserInfoUtil.parseHeader(request), file, daId);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "导入模板下载", notes = "退化分析")
    @GetMapping("/import-template")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public void importTemplate(HttpServletResponse response) throws IOException {
        ewbDaDataDetailService.importTemplate(response);
    }

    //    @ApiOperation(value = "调用画图API", notes = "退化分析")
//    @GetMapping("/getResultPlot")
//    @ApiImplicitParams(value = {
//            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
//    })
    public ServerResponse getResultPlot(@RequestParam("daId") String daId) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getResultPlot(daId));
    }

    @ApiOperation(value = "获取退化数据（单元数量、数据量、测量值）", notes = "退化分析")
    @GetMapping("/getDataInfo")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getDataInfo(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.getDataInfo(daId));
    }

    @ApiOperation(value = "计算模型向导", notes = "退化分析")
    @GetMapping("/calc")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse calc(@NotBlank(message = "DataIdNotEmpty") String daId) throws Exception {
        return ServerResponse.createBySuccess(ewbDaDataDetailService.calc(daId));
    }

    @ApiOperation(value = "选择模型向导", notes = "退化分析")
    @GetMapping("/selectModel")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse selectModel(@NotBlank(message = "DataIdNotEmpty") String daId, @RequestParam("model") String model) {
        ewbDaSettingService.selectModel(daId, model);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "创建寿命数据分析", notes = "退化分析")
    @PostMapping("/createWeibull")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse createWeibull(HttpServletRequest request, @RequestBody EwbDegradationToWeibull param) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        String analysisId = UUIDUtil.getUuid();
        EwbAnalysisReqVo analysis = ewbAnalysisService.createAnalysis(userInfo, param.getWbId(), analysisId, param.getAnalysisName(), null, null, null);
        // 更新设置
        param.getSetting().setAnalysisId(analysisId);
        param.getSetting().setDataId(analysis.getDataId());
        param.getSetting().setSettingId(analysis.getSettingId());
        ewbAnalysisSettingService.saveSetting(userInfo, param.getSetting());
        // 更新数据
        param.getDataList().setAnalysisId(analysisId);
        param.getDataList().setDataId(analysis.getDataId());
        if (param.getDataList().getHotDataChange() == null) {
            param.getDataList().setHotDataChange(2);
        }
        if (param.getDataList().getHotDataRankChange() == null) {
            param.getDataList().setHotDataRankChange(0);
        }
        if (StringUtils.isBlank(param.getDataList().getTimeForRank())) {
            param.getDataList().setTimeForRank(EwbConstants.FLAG_NO_CHECKED);
        }
        ewbAnalysisDataDetailService.saveData(userInfo, param.getDataList(), false);
        return ServerResponse.createBySuccess(analysis);
    }

    @ApiOperation(value = "复制", notes = "退化分析")
    @GetMapping("/copy")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse copy(@NotBlank(message = "DataIdNotEmpty") String daId) {
        return ServerResponse.createBySuccess(ewbDaInfoService.copy(daId));
    }

    @ApiOperation(value = "导出", notes = "分析数据")
    @GetMapping("/export")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public void export(HttpServletResponse response, HttpServletRequest request, @NotBlank(message = "DataIdNotEmpty") String daId) {
        ewbDaDataDetailService.export(response, daId, UserInfoUtil.parseLk(request));
    }
}
