package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.dto.EwbAnalysisNevadaPlotVo;
import com.nationrel.weibull.domain.dto.EwbNevadaSettingDto;
import com.nationrel.weibull.domain.entity.EwbNonNevadaSetting;
import com.nationrel.weibull.domain.vo.*;
import com.nationrel.weibull.service.EwbNevadaService;
import com.nationrel.weibull.service.EwbNonNevadaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

@Api(tags = "内华达分析（整合nevada和nonnevada）")
@Validated
@RestController
@RequestMapping("/nevada")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbNevadaController {

    private final EwbNevadaService nevadaService;
    private final EwbNonNevadaService nonNevadaService;

    @ApiOperation(value = "内华达创建（整合nevada和nonnevada）", notes = "内华达分析（整合nevada和nonnevada）")
    @PostMapping("")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse create(HttpServletRequest request, @RequestBody @Valid NevadaCreateVo param) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return "nonnevada".equals(param.getWhichTree()) ? nonNevadaService.create(param, userInfo) : nevadaService.create(param, userInfo);
    }

    @ApiOperation(value = "内华达列表", notes = "内华达分析")
    @GetMapping("")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse list(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return nonNevadaService.list(wbId);
    }

    @ApiOperation(value = "内华达数据更新", notes = "内华达数据更新")
    @PutMapping("")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse<NevadaDataVo> editNevadaData(HttpServletRequest request, @RequestBody NevadaSettingVo nevadaSetting) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return "nonnevada".equals(nevadaSetting.getWhichTree()) ? nonNevadaService.editNevadaData(nevadaSetting, false, userInfo) : nevadaService.editNevadaData(nevadaSetting, false, userInfo);
    }

    @ApiOperation(value = "内华达页面初始化获取数据（整合nevada和nonnevada）", notes = "内华达页面初始化获取数据（整合nevada和nonnevada）")
    @GetMapping("/init")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse<NevadaDataVo> getNevadaData(HttpServletRequest request, @NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @RequestParam(value = "whichTree") String whichTree) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return "nonnevada".equals(whichTree) ? nonNevadaService.editNevadaData(NevadaSettingVo.builder().nevadaId(nevadaId).build(), true, userInfo) : nevadaService.editNevadaData(NevadaSettingVo.builder().nevadaId(nevadaId).build(), true, userInfo);
    }

    @ApiOperation(value = "内华达表格数据保存", notes = "内华达表格数据保存")
    @PutMapping("/table")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveNevadaTable(HttpServletRequest request, @RequestBody NevadaSaveVo nevadaData) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return "nonnevada".equals(nevadaData.getWhichTree()) ? nonNevadaService.saveNevadaTable(userInfo, nevadaData) : nevadaService.saveNevadaTable(userInfo, nevadaData);
    }

    @ApiOperation(value = "内华达表格数据保存", notes = "内华达表格数据保存")
    @PutMapping("/kpitable")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse saveKpiNevadaTable(HttpServletRequest request, @RequestBody NevadaKpiResultVo nevadaData) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return nevadaService.saveKpiNevadaTable(userInfo, nevadaData);
    }

    @ApiOperation(value = "内华达表格数据保存", notes = "内华达表格数据保存")
    @PutMapping("/validNevada")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "nevadaId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse validNevada(HttpServletRequest request, @NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @NotNull(message = "DataNotEmpty") Integer action) {
        return nonNevadaService.validData(nevadaId, action, UserInfoUtil.parseLk(request));
    }

    @ApiOperation(value = "获取内华达表格数据", notes = "获取内华达表格数据")
    @GetMapping("/table")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "laims per验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getNevadaTableData(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId, @RequestParam(value = "claimsPer", required = false) Integer claimsPer) {
        return nevadaService.getNevadaTableData(nevadaId, claimsPer);
    }

    @ApiOperation(value = "获取内华达表格数据", notes = "获取内华达表格数据")
    @GetMapping("/kpitable")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "laims per验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse getKpiNevadaTableData(@NotBlank(message = "AnalysisIdNotEmpty") String nevadaId) {
        return nevadaService.getKpiNevadaTableData(nevadaId);
    }

    @ApiOperation(value = "更新kpi内华达设置数据", notes = "更新kpi内华达设置数据")
    @PutMapping("/kpiSetting")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse saveKpiNevadaSetting(@RequestBody NevadaKpiResultVo resultVo) {
        return nevadaService.saveKpiNevadaSetting(resultVo);
    }

    @ApiOperation(value = "内华达预测数据", notes = "内华达预测数据")
    @GetMapping("/forecast")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "nevadaId", value = "内华达id", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "forecast", value = "预测数量", dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "forecastType", value = "预测方式", dataType = "int", paramType = "query"),
    })
    public ServerResponse forecast(HttpServletRequest request,
                                   @NotBlank(message = "AnalysisIdNotEmpty") String nevadaId,
                                   @RequestParam(value = "whichTree") String whichTree,
                                   @RequestParam(value = "forecast", defaultValue = "0") Integer forecast,
                                   @RequestParam(value = "forecastType", defaultValue = "0") Integer forecastType,
                                   @RequestParam(value = "boundSided", defaultValue = "00") String boundSided,
                                   @RequestParam(value = "confidenceLevel", defaultValue = "90") Double confidenceLevel,
                                   @RequestParam(value = "confidenceBound", defaultValue = "0") Integer confidenceBound) throws Exception {
        return nonNevadaService.forecastData(whichTree, nevadaId, forecast, forecastType, boundSided, confidenceLevel, confidenceBound, UserInfoUtil.parseHeader(request));
    }

    @ApiOperation(value = "内华达预测数据导出", notes = "内华达预测数据导出")
    @PostMapping("/forecastExport")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
    })
    public void forecastExport(HttpServletResponse response,
                               @RequestBody NevadaDataVo nevadaDataVo) {
        nonNevadaService.forecastExport(nevadaDataVo, response);
    }

    @ApiOperation(value = "删除内华达数据", notes = "删除内华达数据")
    @DeleteMapping("/{nevadaId}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse<NevadaDataVo> delNevadaData(@PathVariable(value = "nevadaId") String nevadaId) {
        return nonNevadaService.delNevadaData(nevadaId);
    }

    @ApiOperation(value = "内华达数据复制", notes = "内华达数据复制")
    @PostMapping("/copy")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "nevadaId", value = "内华达ID", dataType = "String", paramType = "query"),
    })
    public ServerResponse copyNevada(HttpServletRequest request, @RequestBody @Valid EwbNonNevadaSetting nevadaSetting) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        nonNevadaService.copyNevada(userInfo, nevadaSetting.getNevadaId());
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "内华达图名称修改", notes = "内华达图名称修改")
    @PutMapping("/name")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse editNevada(@RequestBody EwbNevadaSettingDto nevadaSetting) {
        return nonNevadaService.editNevada(nevadaSetting);
    }

    @ApiOperation(value = "概率图", notes = "作图")
    @PostMapping("/probabilityPlot")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse probabilityPlot(HttpServletRequest request, @RequestBody @Valid EwbAnalysisNevadaPlotVo param) {
        HeaderUserInfo user = UserInfoUtil.parseHeader(request);
        EwbProbabilityPlotVo plotVo = nevadaService.probabilityPlot(user, param);
        return ServerResponse.createBySuccess(plotVo);
    }
}
