package com.nationrel.weibull.controller;

import com.github.pagehelper.PageInfo;
import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.vo.EwbAnalysisInfoCreateVo;
import com.nationrel.weibull.domain.vo.EwbAnalysisInfoSaveVo;
import com.nationrel.weibull.domain.vo.EwbAnalysisListVo;
import com.nationrel.weibull.service.EwbAnalysisInfoService;
import com.nationrel.weibull.service.EwbAnalysisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

/**
 * @author Aimee on 2020/10/19.
 */
@Api(tags = "寿命数据分析-项目部分")
@Validated
@RestController
@RequestMapping("/project")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbProjectController {
    private final EwbAnalysisService analysisService;
    private final EwbAnalysisInfoService ewbAnalysisInfoService;

    @ApiOperation(value = "项目创建", notes = "项目")
    @PostMapping("/create")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse projectCreate(HttpServletRequest request, @RequestBody @Valid EwbAnalysisInfoCreateVo param) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        return ewbAnalysisInfoService.createAnalysisInfo(userInfo, param.getWbName(), param.getObjectCode(), param.getObjectName(), param.getDescription(), param.getAnalysisType().replaceAll("/lifeData/", ""));
    }

    @ApiOperation(value = "项目删除", notes = "项目")
    @GetMapping("/delete")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目编号", dataType = "String", paramType = "query")
    })
    public ServerResponse delete(@NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        ewbAnalysisInfoService.delete(wbId);
        return ServerResponse.createBySuccess();
    }

    @ApiOperation(value = "项目列表", notes = "项目")
    @GetMapping("/list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "analysisName", value = "分析名称", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "analysisType", value = "分析类型(life:综合分析;weibull:weibull分析;degradation:退化分析;costOpti:成本优化分析;nevadaNRI:内华达分析NRI;nevadaRI:内华达分析RI;)", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageIndex", value = "当前页码", dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "每页条数", dataType = "int", paramType = "query")
    })
    public ServerResponse<PageInfo<EwbAnalysisListVo>> projectList(HttpServletRequest request,
                                                                   @RequestParam(required = false) String analysisName,
                                                                   @Param("analysisType") String analysisType,
                                                                   @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
                                                                   @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        HeaderUserInfo userInfo = UserInfoUtil.parseHeader(request);
        analysisType = analysisType.replace("/lifeData/", "");
        PageInfo<EwbAnalysisListVo> pageResult = ewbAnalysisInfoService.listByPage(analysisName, userInfo, analysisType, pageIndex, pageSize);
        return ServerResponse.createBySuccess(pageResult);
    }

    @ApiOperation(value = "项目列表", notes = "项目")
    @GetMapping("/statistic")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse statistic(HttpServletRequest request) {
        return analysisService.statistic(UserInfoUtil.parseHeader(request));
    }

    @ApiOperation(value = "项目编辑保存", notes = "项目")
    @PostMapping("/save")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse projectSave(HttpServletRequest request, @RequestBody @Valid EwbAnalysisInfoSaveVo param) {
        return ewbAnalysisInfoService.editProject(UserInfoUtil.parseHeader(request), param.getWbId(), param.getWbName(), param.getObjectCode(), param.getObjectName(), param.getDescription());
    }

}
