package com.nationrel.weibull.controller;

import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.util.common.UserInfoUtil;
import com.nationrel.weibull.domain.vo.EwbStressStrengthCreateVo;
import com.nationrel.weibull.domain.vo.EwbStressStrengthEditNameVo;
import com.nationrel.weibull.domain.vo.EwbStressStrengthEditVo;
import com.nationrel.weibull.domain.vo.EwbStressStrengthVo;
import com.nationrel.weibull.service.EwbStressStrengthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.List;

/**
 * ss曲线分析
 */
@Api(tags = "ss曲线分析")
@Validated
@RestController
@RequestMapping("/stress")
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbStressStrengthController {

    private final EwbStressStrengthService stressStrengthService;

    @ApiOperation(value = "ss曲线分析列表", notes = "ss曲线分析")
    @GetMapping("/list")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query")
    })
    public ServerResponse<List<EwbStressStrengthVo>> analysisList(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId) {
        return stressStrengthService.listByPage(wbId);
    }

    @ApiOperation(value = "分析列表", notes = "多图")
    @GetMapping("/analysisList")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "wbId", value = "项目ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "analysisId", value = "分析ID", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "distributingCode", value = "分布类型", dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "type", value = "应力强度类型", dataType = "String", paramType = "query")
    })
    public ServerResponse analysisList(@RequestParam @NotBlank(message = "AnalysisIdNotEmpty") String wbId, @RequestParam(required = false) String analysisId, @RequestParam @NotBlank(message = "DistributingCodeNotEmpty") String distributingCode, @RequestParam @NotBlank(message = "StressStrengthNotEmpty") String type) {
        return stressStrengthService.selectAnalysisList(wbId, analysisId, distributingCode, type);
    }

    @ApiOperation(value = "ss曲线分析创建", notes = "ss曲线分析")
    @PostMapping("")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse create(HttpServletRequest request, @RequestBody @Valid EwbStressStrengthCreateVo param) {
        return ServerResponse.createBySuccess(stressStrengthService.create(param.getName(), param.getWbId(), UserInfoUtil.parseHeader(request)));
    }

    @ApiOperation(value = "ss曲线分析更新", notes = "ss曲线分析更新")
    @PutMapping("")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse edit(@RequestBody @Valid EwbStressStrengthEditVo entity) {
        return stressStrengthService.edit(entity);
    }

    @ApiOperation(value = "ss曲线分析更新名称", notes = "ss曲线分析更新名称")
    @PutMapping("/name")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header")
    })
    public ServerResponse editName(@RequestBody @Valid EwbStressStrengthEditNameVo entity) {
        return stressStrengthService.editName(entity);
    }

    @ApiOperation(value = "ss曲线单条信息获取", notes = "ss曲线单条信息获取")
    @GetMapping("/{id}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "id", value = "分析ID", dataType = "String", paramType = "query")
    })
    public ServerResponse create(HttpServletRequest request, @PathVariable(value = "id") String id) {
        return stressStrengthService.getById(id);
    }

    @ApiOperation(value = "ss曲线单条信息删除", notes = "ss曲线单条信息删除")
    @DeleteMapping("/{id}")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "Authorization", value = "验证信息", dataType = "String", paramType = "header"),
            @ApiImplicitParam(name = "id", value = "分析ID", dataType = "String", paramType = "query")
    })
    public ServerResponse delete(HttpServletRequest request, @PathVariable(value = "id") String id) {
        return stressStrengthService.deleteById(id);
    }
}
