package com.nationrel.weibull.domain.vo;

import com.nationrel.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.general.exception.DescribeException;
import com.nationrel.general.domain.enums.common.EwbConstants;
import com.nationrel.weibull.domain.entity.EwbAnalysisDataDetail;
import com.nationrel.weibull.util.WeibullNumberUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Data
@SuperBuilder
@AllArgsConstructor
@NoArgsConstructor
public class SolveFixEffectivinessVo {

    @ApiModelProperty(value = "计算ID")
    private String solveId;

    @ApiModelProperty(value = "计算名称")
    private String solveName;

    @ApiModelProperty(value = "表格数据是否修标识 1:修改过")
    private String isUpdateDefectTableData;

    @ApiModelProperty(value = "表格数据是否修标识 1:修改过")
    private String isUpdateFixTableData;

    @ApiModelProperty(value = "修正后数据id")
    private String fixDataId;

    @ApiModelProperty(value = "有缺陷数据id")
    private String defectdataId;

    @ApiModelProperty(value = "页面明细数据集id")
    private String dataId;

    @ApiModelProperty(value = "当前页起始数据编号")
    private int fixStartDataNo;

    @ApiModelProperty(value = "当前页结束数据编号")
    private int fixEndDataNo;

    @ApiModelProperty(value = "当前页起始数据编号")
    private int defectStartDataNo;

    @ApiModelProperty(value = "当前页结束数据编号")
    private int defectEndDataNo;

    @ApiModelProperty(value = "验证类型")
    private Integer verficationCode;

    @ApiModelProperty(value = "估计方法")
    private String estimationMethodCode;

    @ApiModelProperty(value = "单位")
    private String unitCode;

    @ApiModelProperty(value = "页面明细数据集合")
    private List<String[]> fixDatas;

    @ApiModelProperty(value = "页面明细数据集合")
    private List<String[]> dataDetails;

    @ApiModelProperty(value = "页面明细数据集合")
    private List<String[]> defectDatas;

    private int totalNumFailureAndSuspensions;

    private int totalNumSuspensions;

    private int totalNumFailures;

    private int totalNumOfRows;

    private int totalFailureRows;

    private int totalSuspensionRows;

    public SolveFixEffectivinessVo(int defectStartDataNo, int defectEndDataNo, int fixStartDataNo, int fixEndDataNo, List<String[]> dataDetails, List<String[]> defectDatas, List<String[]> fixDatas, int totalNumFailureAndSuspensions, int totalNumSuspensions, int totalNumFailures, int totalNumOfRows, int totalFailureRows, int totalSuspensionRows) {
        this.defectStartDataNo = defectStartDataNo;
        this.defectEndDataNo = defectEndDataNo;
        this.fixStartDataNo = fixStartDataNo;
        this.fixEndDataNo = fixEndDataNo;
        this.dataDetails = dataDetails;
        this.defectDatas = defectDatas;
        this.fixDatas = fixDatas;
        this.totalNumFailureAndSuspensions = totalNumFailureAndSuspensions;
        this.totalNumSuspensions = totalNumSuspensions;
        this.totalNumFailures = totalNumFailures;
        this.totalNumOfRows = totalNumOfRows;
        this.totalFailureRows = totalFailureRows;
        this.totalSuspensionRows = totalSuspensionRows;
    }

    /**
     * 解析修正后的数据
     *
     * @param param
     * @return
     */
    public static List<EwbAnalysisDataDetail> parseFixDatas(SolveFixEffectivinessVo param, Integer languageKey) {
        List<EwbAnalysisDataDetail> result = new ArrayList<>();
        try {
            for (int i = 0; i < param.getFixEndDataNo(); i++) {
                String[] fixData = param.getFixDatas().get(i);
                // 0,1分别代表开始结束时间，此两项为必填数据
                if (WeibullNumberUtil.isNull(Double.parseDouble(fixData[0])) || WeibullNumberUtil.isNull(Double.parseDouble(fixData[1]))) {
                    throw new DescribeException((i + 1), EwbCustomPromptEnum.填写正确的开始结束时间, languageKey);
                }
                Double start = WeibullNumberUtil.parseDouble(fixData[0]);
                Double end = WeibullNumberUtil.parseDouble(fixData[1]);
                if (end - start <= 0) {
                    throw new DescribeException((i + 1), EwbCustomPromptEnum.结束时间必须大于开始时间, languageKey);
                } else {
                    // 计算时间数据，当前的计算规则为结束时间-开始时间
                    if (WeibullNumberUtil.isNull(Double.parseDouble(fixData[3]))) {
                        fixData[2] = new BigDecimal(end.toString()).subtract(new BigDecimal(start.toString())).toString();
                    }
                    result.add(EwbAnalysisDataDetail.builder().failureEndTime(end - start).quantity(1).failureSuspension("").lastInspectTime(0.0).build());
                }
            }
        } catch (NumberFormatException e) {
            throw new DescribeException(EwbCustomPromptEnum.开始结束时间必须为数字, languageKey);
        }

        return result;
    }

    /**
     * 解析有缺陷的数据
     *
     * @param param
     * @return
     */
    public static List<EwbAnalysisDataDetail> parseDefectDatas(SolveFixEffectivinessVo param, Integer languageKey) {
        List<EwbAnalysisDataDetail> result = new ArrayList<>();
        try {
            Integer totalNumSuspensions = 0;
            for (int i = 0; i < param.getDefectEndDataNo(); i++) {
                String[] data = param.getDefectDatas().get(i);
                totalNumSuspensions = EwbConstants.STATE_SUSPENSION.equals(data[1].trim()) ? ++totalNumSuspensions : totalNumSuspensions;
                // 0,1分别代表开始结束时间，此两项为必填数据
                if (WeibullNumberUtil.isNull(Double.parseDouble(data[0])) || StringUtils.isBlank(data[1]) || (WeibullNumberUtil.isNull(Double.parseDouble(data[2])))) {
                    throw new DescribeException((i + 1), EwbCustomPromptEnum.填写正确的开始结束时间, languageKey);
                } else {
                    Double start = WeibullNumberUtil.parseDouble(data[0]);
                    Double end = WeibullNumberUtil.parseDouble(data[2]);
                    if (end - start < 0) {
                        throw new DescribeException((i + 1), EwbCustomPromptEnum.结束时间必须大于开始时间, languageKey);
                    }
                    if (EwbConstants.STATE_FAILURE.equalsIgnoreCase(data[1])) {
                        param.setTotalNumFailures(param.getTotalNumFailures() + 1);
                    } else {
                        param.setTotalNumSuspensions(param.getTotalNumSuspensions() + 1);
                    }
                    // 计算时间数据，当前的计算规则为结束时间-开始时间
                    if (WeibullNumberUtil.isNull(Double.parseDouble(data[3]))) {
                        data[3] = new BigDecimal(end.toString()).subtract(new BigDecimal(start.toString())).toString();
                    }
                    result.add(EwbAnalysisDataDetail.builder().failureEndTime(end - start).quantity(1).failureSuspension(data[1]).lastInspectTime(0.0).build());
                }
            }
            param.setTotalNumSuspensions(totalNumSuspensions);
            param.setTotalNumFailures(param.getTotalNumFailureAndSuspensions() - totalNumSuspensions);
            param.setTotalSuspensionRows(totalNumSuspensions);
            param.setTotalFailureRows(param.getTotalNumFailures());
        } catch (NumberFormatException e) {
            throw new DescribeException(EwbCustomPromptEnum.开始结束时间必须为数字, languageKey);
        }

        return result;
    }
}
