package com.nationrel.weibull.service;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.nationrel.general.common.ServerResponse;
import com.nationrel.general.domain.auth.HeaderUserInfo;
import com.nationrel.general.domain.enums.common.WhetherFlagEnum;
import com.nationrel.general.util.common.BasicDataUtil;
import com.nationrel.general.util.common.UUIDUtil;
import com.nationrel.weibull.dao.*;
import com.nationrel.weibull.domain.dto.EwbNevadaCreateDto;
import com.nationrel.weibull.domain.entity.EwbAnalysisInfo;
import com.nationrel.weibull.domain.entity.EwbNonNevadaSetting;
import com.nationrel.general.domain.enums.weibull.EwbCommonUsedEnum;
import com.nationrel.general.domain.enums.weibull.EwbCustomPromptEnum;
import com.nationrel.weibull.domain.enums.EwbConfidenceBoundEnum;
import com.nationrel.weibull.domain.enums.EwbConfidenceTypeCodeEnum;
import com.nationrel.weibull.domain.enums.EwbNevadaForecastTypeEnum;
import com.nationrel.weibull.domain.vo.EwbAnalysisListVo;
import com.nationrel.weibull.domain.vo.EwbAnalysisReqVo;
import com.nationrel.weibull.domain.vo.NevadaCreateVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author Aimee on 2020/9/15.
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class EwbAnalysisInfoService {

    private final EwbAnalysisInfoMapper analysisInfoMapper;
    private final EwbNonNevadaDataMapper nonNevadaDataMapper;
    private final EwbNonNevadaSettingMapper nonNevadaSettingMapper;
    private final EwbAnalysisService analysisService;
    private final EwbDaInfoService daInfoService;

    @Transactional(rollbackFor = Exception.class)
    public ServerResponse createAnalysisInfo(HeaderUserInfo userInfo, String wbName, String objectCode, String objectName, String description, String analysisType){

        // 创建项目信息
        String wbId = UUIDUtil.getUuid();
        EwbAnalysisInfo analysisInfo = EwbAnalysisInfo.builder().wbId(wbId)
                .wbName(wbName)
                .objectCode(objectCode)
                .analysisType(analysisType)
                .objectName(objectName)
                .description(description)
                .insertTime(new Date())
                .insertUserId(userInfo.getUserId())
                .updateTime(new Date())
                .versionNo(0)
                .delFlag("0").build();
        analysisInfoMapper.insert(analysisInfo);

        // 创建分析信息
        String analysisId = UUIDUtil.getUuid();
        switch (analysisType) {
            case "life":
            case "weibull":
                analysisService.createAnalysis(userInfo, wbId, analysisId, "分析1", null, null, null);
                break;
            case "degradation":
                daInfoService.create(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1", wbId, userInfo);
                break;
            case "costOpti":
                //daInfoService.create("分析1", wbId, userInfo);
                break;
            case "nevadaNRI":
                EwbNevadaCreateDto createDto = EwbNonNevadaService.createDto(NevadaCreateVo.builder().wbId(wbId).analysisName(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1").build(), userInfo);
                EwbNonNevadaSetting nevadaSetting = createDto.getNonNevadaSetting();
                EwbAnalysisReqVo analysis = analysisService.createAnalysis(userInfo, wbId, nevadaSetting.getNevadaId(), "", null, null, null);
                nevadaSetting.setDataId(analysis.getDataId());
                nevadaSetting.setSettingId(analysis.getSettingId());
                nevadaSetting.setForecastType(EwbNevadaForecastTypeEnum.未设置.getIndex());
                nevadaSetting.setConfidenceBound(EwbConfidenceBoundEnum.双侧.getIndex());
                nevadaSetting.setConfidenceLevel(EwbConfidenceBoundEnum.双侧.getConfidenceLevel());
                nevadaSetting.setBoundSided(EwbConfidenceTypeCodeEnum.无.getIndex());
                nevadaSetting.setIsComplete(WhetherFlagEnum.NO.getIndex());
                nonNevadaSettingMapper.insert(nevadaSetting);
                nonNevadaDataMapper.insertBatch(createDto.getNonNevadaData());
                break;
            case "nevadaRI":
                EwbNevadaService.createDto(NevadaCreateVo.builder().wbId(wbId).analysisName(EwbCommonUsedEnum.getLanguageMessageByKey(EwbCommonUsedEnum.分析, userInfo.getLanguageKey()) + "1").nevadaType("nevadaRI").build(), userInfo);
                break;
            default:
                break;
        }

        return ServerResponse.createBySuccess(analysisInfo);
    }

    public PageInfo<EwbAnalysisListVo> listByPage(String analysisName, HeaderUserInfo userInfo, String analysisType, int pageIndex, int pageSize) {
        PageHelper.startPage(pageIndex, pageSize);
        analysisName = BasicDataUtil.parseEmptyStr(analysisName);
        List<EwbAnalysisListVo> list = analysisInfoMapper.selectSearch(userInfo.getUserId(), analysisName, userInfo.getUserName(), analysisType);
        return new PageInfo<>(list);
    }

    @Transactional(rollbackFor = Exception.class)
    public ServerResponse editProject(HeaderUserInfo userInfo, String wbId, String wbName, String objectCode, String objectName, String description) {
        EwbAnalysisInfo analysisInfo = analysisInfoMapper.selectById(wbId);
        if (analysisInfo == null) {
            return ServerResponse.createByErrorMessage(EwbCustomPromptEnum.项目信息不存在, userInfo.getLanguageKey());
        }
        if (!wbName.equals(analysisInfo.getWbName()) || !objectCode.equals(analysisInfo.getObjectCode()) || !objectName.equals(analysisInfo.getObjectName()) || !description.equals(analysisInfo.getDescription())) {
            EwbAnalysisInfo newEwbAnalysis = EwbAnalysisInfo.builder()
                    .wbId(wbId)
                    .wbName(wbName)
                    .objectCode(objectCode)
                    .objectName(objectName)
                    .description(description)
                    .updateTime(new Date())
                    .updateUserId(userInfo.getUserId())
                    .updateTime(new Date())
                    .delFlag("0").build();
            return ServerResponse.createBySuccess(analysisInfoMapper.updateById(newEwbAnalysis));
        } else {
            return ServerResponse.createBySuccess();
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void delete(String wbId) {
        analysisInfoMapper.deleteWb(wbId);
    }

    /**
     * 更新修改者信息
     * @param userId
     * @param analysisId
     */
    public static void updateUpdateInfo(Integer userId, String analysisId, EwbAnalysisInfoMapper analysisInfoMapper, EwbAnalysisMapper analysisMapper) {
        String wbId = analysisMapper.selectWbIdByAnalysisId(analysisId);
        analysisInfoMapper.updateUpdateInfo(new Date(), userId, wbId);
    }
}
